<?php

namespace Drupal\todoist_api\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\todoist_api\Rest\Client;

/**
 * Provide a list of tasks.
 */
class TodoistapiTasks extends ControllerBase {

  /**
   * Guzzle\Client instance.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Rest cliet instance Todoist_api.
   *
   * @var \todoist_api\Rest\Client
   */
  protected $todoistclient;

  /**
   * Config Factory Interface.
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configfactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $configfactory, ClientInterface $http_client, Client $todoistclient) {
    $this->configfactory = $configfactory->getEditable('todoist_api.settings');
    $this->httpClient = $http_client;
    $this->todoistclient = $todoistclient;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('http_client'),
      $container->get('todoist_api.client')
    );
  }

  /**
   * Posts route callback.
   *
   * @param int $limit
   *   The total number of posts we want to fetch.
   * @param string $sort
   *   The sorting order.
   *
   * @return array
   *   A render array used to show the Posts list.
   */

  /**
   * A simple controller method to explain what the tablesort example is about.
   */
  public function build() {
    if (empty($this->configfactory->get('api_endpoints')) || empty($this->configfactory->get('access_token'))) {
      \Drupal::messenger()->addError($this->t('To access your task list first, you must set your Access token.'));
      return $this->redirect('todoist_api.configform');
    }

    // We are going to output the results in a table with a nice header.
    $header = [
      // The header gives the table the information it needs in order to make
      // the query calls for ordering. TableSort uses the field information
      // to know what database column to sort by.
      ['data' => $this->t('Title of the task'), 'field' => 'content'],
      ['data' => $this->t('Description'), 'field' => 'description'],
      ['data' => $this->t('Task added date'), 'field' => 'created_at'],
      ['data' => $this->t('Task due date'), 'field' => 'due'],
      ['data' => $this->t('Complete status'), 'field' => 'is_completed'],
      ['data' => $this->t('Action'), 'field' => 'created_at', 'colspan' => 3],
    ];

    // Get end-point Url.
    $end_point = 'tasks';

    // Fetch data.
    $response_data = $this->todoistclient->get($end_point);

    // Prepare table rows.
    $rows = [];
		foreach ($response_data as $row) {
      $createdDate = \Drupal::service('date.formatter')->format((new \DateTime($row['created_at']))->getTimestamp(), 'custom', 'D, d M Y h:i a');
      $dueDate = \Drupal::service('date.formatter')->format((new \DateTime($row['due']['datetime']))->getTimestamp(), 'custom', 'D, d M Y h:i a');

			$complete_link = $this->linkgenerater($row['id'], 'close');
      $edit_link = $this->linkgenerater($row['id'], 'edit');
      $delete_link = $this->linkgenerater($row['id'], 'delete');
      $rows[] = [
        'data' => [
          $row['content'],
          !empty($row['description']) ? $row['description'] : $this->t('No description.'),
          isset($row['created_at']) ? $createdDate : $this->t('No date'),
          isset($row['due']) ? $dueDate : $this->t('No date'),
          $row['is_completed'] ? 'Complete' : 'Uncomplete',
          $complete_link,
          $edit_link,
          $delete_link,
        ],
      ];
    }

    // Build the link to add new task.
    $build['addtask'] = [
      '#type' => 'link',
      '#url' => Url::fromRoute('todoist_api.addtask'),
      '#title' => $this->t('Add New Task'),
      '#attributes' => [
        'class' => [
          'button',
          'button-action',
          'button--small',
          'button--primary',
        ],
      ],
    ];

    // Build the table for the nice output.
    $build['taskssort_table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      // Optional to indicate whether the table headers should be sticky.
      '#sticky' => TRUE,
      // Optional empty text for the table if resultset is empty.
      '#empty' => $this->t('No task created...'),

    ];

    return $build;
  }

  /**
   * Function return button link for edit, delte and close task.
   */
  protected function linkgenerater($id, $type) {
    switch ($type) {
      case 'close':
        $link_title = $this->t('Complete');
        $class = 'button--primary';
        break;

      case 'delete':
        $link_title = $this->t('Delete');
        $class = 'button--danger';
        break;

      case 'edit':
        $link_title = $this->t('Edit');
        $class = 'button--primary';
        break;
    }
    $url = Url::fromUri('internal:/admin/content/todoist/' . $id . '/' . $type);
    $link = Link::fromTextAndUrl($link_title, $url);
    $link = $link->toRenderable();
    $link['#attributes'] = [
      'class' => [
        'button',
        'button-action',
        'button--small',
        $class,
      ],
    ];
    return \Drupal::service('renderer')->render($link);
  }

}
